@@ -2,7 +2,7 @@ import numpy as np
 from matplotlib import colors
 import matplotlib.pyplot as plt
 from matplotlib.ticker import LogLocator
-from datetime import datetime, timedelta
+from datetime import date, datetime, timedelta
 import matplotlib.dates as dts
 import pandas as pd
 import os
@@ -39,12 +39,12 @@ __pdoc__ = {
 
 # The final geometric mean diameters of diameter and mobility bins
 dp_ion = np.array([7.86360416e-10, 9.08232168e-10, 1.04902018e-09, 1.21167006e-09,
-       1.39958930e-09, 1.61672083e-09, 1.86762862e-09, 2.15759741e-09,
-       2.49274932e-09, 2.88018000e-09, 3.32811839e-09, 3.84611427e-09,
-       4.44525917e-09, 5.13844742e-09, 5.94068566e-09, 6.86946146e-09,
-       7.94518431e-09, 9.19171623e-09, 1.06370142e-08, 1.23139134e-08,
-       1.42610904e-08, 1.65242568e-08, 1.91576555e-08, 2.22259544e-08,
-       2.58066722e-08, 2.99933244e-08, 3.48995548e-08, 4.06646353e-08])*1e9
+1.39958930e-09, 1.61672083e-09, 1.86762862e-09, 2.15759741e-09,
+2.49274932e-09, 2.88018000e-09, 3.32811839e-09, 3.84611427e-09,
+4.44525917e-09, 5.13844742e-09, 5.94068566e-09, 6.86946146e-09,
+7.94518431e-09, 9.19171623e-09, 1.06370142e-08, 1.23139134e-08,
+1.42610904e-08, 1.65242568e-08, 1.91576555e-08, 2.22259544e-08,
+2.58066722e-08, 2.99933244e-08, 3.48995548e-08, 4.06646353e-08])*1e9
 
 dp_par = np.array([7.498942093324539870e-01,8.659643233600640144e-01,
 9.999999999999980016e-01,1.154781984689456031e+00,1.333521432163321974e+00,
@@ -69,36 +69,36 @@ mob_ion = np.array([3.162277660168379937e-04,2.371373705661659990e-04,
 1.778279410038920042e-07,1.333521432163329868e-07])*1e4
 
 mob_ion_geomeans=np.array([2.73841963e-04, 2.05352503e-04, 1.53992653e-04, 1.15478198e-04,
-       8.65964323e-05, 6.49381632e-05, 4.86967525e-05, 3.65174127e-05,
-       2.73841963e-05, 2.05352503e-05, 1.53992653e-05, 1.15478198e-05,
-       8.65964323e-06, 6.49381632e-06, 4.86967525e-06, 3.65174127e-06,
-       2.73841963e-06, 2.05352503e-06, 1.53992653e-06, 1.15478198e-06,
-       8.65964323e-07, 6.49381632e-07, 4.86967525e-07, 3.65174127e-07,
-       2.73841963e-07, 2.05352503e-07, 1.53992653e-07])*1e4
+8.65964323e-05, 6.49381632e-05, 4.86967525e-05, 3.65174127e-05,
+2.73841963e-05, 2.05352503e-05, 1.53992653e-05, 1.15478198e-05,
+8.65964323e-06, 6.49381632e-06, 4.86967525e-06, 3.65174127e-06,
+2.73841963e-06, 2.05352503e-06, 1.53992653e-06, 1.15478198e-06,
+8.65964323e-07, 6.49381632e-07, 4.86967525e-07, 3.65174127e-07,
+2.73841963e-07, 2.05352503e-07, 1.53992653e-07])*1e4
 
 dp_par_geomeans=np.array([0.80584219,  0.93057204,  1.07460783,  1.24093776,  1.43301257,
-        1.6548171 ,  1.91095297,  2.20673407,  2.54829675,  2.94272718,
-        3.39820833,  3.92418976,  4.53158364,  5.23299115,  6.0429639 ,
-        6.97830585,  8.05842188,  9.30572041, 10.74607828, 12.40937761,
-       14.3301257 , 16.548171  , 19.10952975, 22.06734069, 25.48296748,
-       29.42727176, 33.98208329, 39.24189758])
+1.6548171 ,  1.91095297,  2.20673407,  2.54829675,  2.94272718,
+3.39820833,  3.92418976,  4.53158364,  5.23299115,  6.0429639 ,
+6.97830585,  8.05842188,  9.30572041, 10.74607828, 12.40937761,
+14.3301257 , 16.548171  , 19.10952975, 22.06734069, 25.48296748,
+29.42727176, 33.98208329, 39.24189758])
 
 dlogmob_ion=np.array([0.125, 0.125, 0.125, 0.125, 0.125, 0.125, 0.125, 0.125, 0.125,
-    0.125, 0.125, 0.125, 0.125, 0.125, 0.125, 0.125, 0.125, 0.125,
-    0.125, 0.125, 0.125, 0.125, 0.125, 0.125, 0.125, 0.125, 0.125,
-    0.125])
+0.125, 0.125, 0.125, 0.125, 0.125, 0.125, 0.125, 0.125, 0.125,
+0.125, 0.125, 0.125, 0.125, 0.125, 0.125, 0.125, 0.125, 0.125,
+0.125])
 
 dlogdp_ion = np.array([0.06257524, 0.0625811 , 0.06259375, 0.06260838, 0.06262533,
-    0.06264495, 0.06266769, 0.06269404, 0.06272461, 0.06276008,
-    0.06280128, 0.06284916, 0.06290487, 0.06296974, 0.06304539,
-    0.0631337 , 0.06323696, 0.06335788, 0.06349974, 0.0636665 ,
-    0.06386292, 0.06409481, 0.06436924, 0.06469482, 0.06508209,
-    0.06554394, 0.06609614, 0.06639699])
+0.06264495, 0.06266769, 0.06269404, 0.06272461, 0.06276008,
+0.06280128, 0.06284916, 0.06290487, 0.06296974, 0.06304539,
+0.0631337 , 0.06323696, 0.06335788, 0.06349974, 0.0636665 ,
+0.06386292, 0.06409481, 0.06436924, 0.06469482, 0.06508209,
+0.06554394, 0.06609614, 0.06639699])
 
 dlogdp_par=np.array([0.0625, 0.0625, 0.0625, 0.0625, 0.0625, 0.0625, 0.0625, 0.0625,
-    0.0625, 0.0625, 0.0625, 0.0625, 0.0625, 0.0625, 0.0625, 0.0625,
-    0.0625, 0.0625, 0.0625, 0.0625, 0.0625, 0.0625, 0.0625, 0.0625,
-    0.0625, 0.0625, 0.0625, 0.0625, 0.0625])
+0.0625, 0.0625, 0.0625, 0.0625, 0.0625, 0.0625, 0.0625, 0.0625,
+0.0625, 0.0625, 0.0625, 0.0625, 0.0625, 0.0625, 0.0625, 0.0625,
+0.0625, 0.0625, 0.0625, 0.0625, 0.0625])
 
 filename_formats = [
 ['%Y-%m-%d.ions.nds','%Y-%m-%d.particles.nds','%Y-%m-%d.log'],
@@ -138,8 +138,8 @@ def make_config():
 
     # Collect data from the user
     print()
-    print("Enter name of configuration file.")
-    print("E.g. ./configs/campaign.yml")
+    print("Enter name of configuration file (full path)")
+    print("E.g. /home/user/campaign.yml")
     while True:
         config_file = input("> ")
         if len(config_file)>0:
@@ -150,8 +150,8 @@ def make_config():
             continue
 
     print()
-    print("Give path(s) to raw data. If multiple paths give them as comma separated list.")
-    print("E.g. /data/nais/2021,/data/nais/2022")
+    print("Give full path(s) to raw data. If multiple paths give them as comma separated list.")
+    print("E.g. /home/user/data/2021,/home/user/data/2022")
     while True:
         user_input = input("> ")
         if len(user_input)>0:
@@ -161,8 +161,8 @@ def make_config():
             continue
 
     print()
-    print("Path to where processed data is saved.")
-    print("E.g. ./data/campaign/processed")
+    print("Full path to where processed data is saved.")
+    print("E.g. /home/user/campaign")
     while True:
         save_path = input("> ")
         if len(save_path)>0:
@@ -197,8 +197,8 @@ def make_config():
             continue
 
     print()
-    print("Enter name of database file")
-    print("E.g. ./logs/campaign.json")
+    print("Enter name of database file (full path)")
+    print("E.g. /home/user/campaign.json")
     while True:
         database_file = input("> ")
         if len(database_file)>0:
@@ -647,31 +647,39 @@ def process_data(
 
             # Temperature
             t_df = 273.15 + df_rec[t_name].astype(float)
-            t_df = t_df.interpolate().values.flatten()
 
             # Pressure
             p_df = 100.0 * df_rec[p_name].astype(float)
-            p_df = p_df.interpolate().values.flatten()
 
             # Sampleflow
             if sf_names is not None:
                 flow_df = df_rec[sf_names].astype(float).sum(axis=1)
-                flow_df = flow_df.interpolate().values.flatten()
             if sf_name is not None:
                 flow_df = df_rec[sf_name].astype(float)
-                flow_df = flow_df.interpolate().values.flatten()
 
-            # If any of the relevant data are all NaNs return nothing
+            # Test if the sampleflow is in cm3/s (old models) or 
+            # l/min and possibly convert to l/min
+            if (np.nanmedian(flow_df)>300):
+                flow_df = (flow_df/1000.0) * 60.0
+            else:
+                pass
+
+            # If all parameters are NaN
+            # e.g. sensor is broken
             if (np.all(np.isnan(flow_df)) |
                 np.all(np.isnan(p_df)) |
                 np.all(np.isnan(t_df))):
                 return None
 
-            # Test if the sampleflow is in cm3/s (old models) or l/min and possibly convert to l/min
-            if flow_df[0]>100:
-                flow_df = (flow_df/1000.0) * 60.0
-            else:
-                pass
+            # Interpolate out NaNs
+            t_df = t_df.interpolate(method="nearest")
+            p_df = p_df.interpolate(method="nearest")
+            flow_df = flow_df.interpolate(method="nearest")
+
+            # Sanity check the values
+            t_df = t_df.where(~((t_df<=223.)|(t_df>=353.)),np.nan).values.flatten()
+            p_df = p_df.where(~((p_df<=37000.)|(p_df>=121000.)),np.nan).values.flatten()
+            flow_df = flow_df.where(~((flow_df<=48.)|(flow_df>=60.)),np.nan).values.flatten()
 
             # Correct the number concentrations to standard conditions (optional)
             if (sealevel_corr):
@@ -862,11 +870,6 @@ def check_config(f):
 
     print(f)
 
-    abs_path_to_f = os.path.abspath(f)
-    a,b = os.path.split(abs_path_to_f)
-    cwd = os.getcwd()
-    os.chdir(a)
-
     with open(f,'r') as stream:
         try:
             config = yaml.safe_load(stream)
@@ -894,21 +897,19 @@ def check_config(f):
 
     if start_date=='':
         pass
+    elif isinstance(start_date,date):
+        pass
     else:
-        try:
-            pd.to_datetime(start_date)
-        except:
-            print("Bad start_date")
-            return False
+        print("Bad start_date")
+        return False
 
     if end_date=='':
         pass
+    elif isinstance(end_date,date):
+        pass
     else:
-        try:
-            pd.to_datetime(end_date)
-        except:
-            print("Bad end_date")
-            return False
+        print("Bad end_date")
+        return False
 
     if os.path.exists(save_path)==False:
         print("Bad save path")
@@ -949,7 +950,6 @@ def check_config(f):
         print("Bad inlet_length")
         return False
 
-    os.chdir(cwd)
     return True
 
 def nais_processor(config_file):
@@ -966,14 +966,8 @@ def nais_processor(config_file):
     if not check_config(config_file):
         return 
 
-    config_abs_path = os.path.abspath(config_file)
-    config_dir,config_fn = os.path.split(config_abs_path)
-    cwd = os.getcwd()
-    os.chdir(config_dir)
-
     # Today
-    today_dt = datetime.today()
-    today = today_dt.strftime('%Y%m%d')
+    today_dt = date.today()
 
     with open(config_file,'r') as stream:
         config = yaml.safe_load(stream)
@@ -989,10 +983,10 @@ def nais_processor(config_file):
         apply_corrections = config['apply_corrections']
         apply_cleaning=config["apply_cleaning"]
 
-    if len(end_date)==0:
-        end_date = today
-    if len(start_date)==0:
-        start_date = "2000-01-01"
+    if start_date=='':
+        start_date = date(2000,1,1)
+    if end_date=='':
+        end_date = today_dt
 
     db = TinyDB(database)
     check = Query()
@@ -1003,46 +997,46 @@ def nais_processor(config_file):
     start_date_str = start_dt.strftime("%Y%m%d")
     end_date_str = end_dt.strftime("%Y%m%d")
 
-    # Convert load and save paths to absolute paths
-    load_path = [os.path.abspath(x) for x in load_path]
-    save_path = os.path.abspath(save_path)
-
     # Make a list of datetimes that the config file is interested in
     list_of_datetimes = pd.date_range(start=start_date_str, end=end_date_str)
 
     # list existing dates
     list_of_existing_dates = [x['timestamp'] for x in db.search(check.diagnostics.exists())]
-    earliest_existing_date = np.min(pd.to_datetime(list_of_existing_dates))
 
     # Add unprocessed datafiles to the database
     for x in list_of_datetimes:
-        if ((x.strftime('%Y%m%d') in list_of_existing_dates) |
-            (x < earliest_existing_date)):
+        if (x.strftime('%Y%m%d') in list_of_existing_dates):
             continue
         else:
             files_found=False
             for z in load_path:
                 for y in filename_formats:
 
-                    if ( (os.path.exists(z+x.strftime(y[0])) | # ions
-                         os.path.exists(z+x.strftime(y[1]))) & # particles
-                         os.path.exists(z+x.strftime(y[2])) # diagnostics
+                    ion_fn = os.path.join(z,x.strftime(y[0]))
+                    particle_fn = os.path.join(z,x.strftime(y[1]))
+                    diagnostic_fn = os.path.join(z,x.strftime(y[2]))
+
+                    if ( (os.path.exists(ion_fn) | # ions
+                         os.path.exists(particle_fn)) & # particles
+                         os.path.exists(diagnostic_fn) # diagnostics
                        ):
 
+                        dtstr = x.strftime('%Y%m%d')
+
                         db.insert(
-                            {'timestamp':x.strftime('%Y%m%d'),
-                            'diagnostics':z+x.strftime(y[2])}
+                            {'timestamp':dtstr,
+                            'diagnostics':diagnostic_fn}
                             )
 
-                        if os.path.exists(z+x.strftime(y[0])):
+                        if os.path.exists(ion_fn):
                             db.update(
-                                {'ions':z+x.strftime(y[0])},
-                                check.timestamp==x.strftime('%Y%m%d'))
+                                {'ions':ion_fn},
+                                check.timestamp==dtstr)
 
-                        if os.path.exists(z+x.strftime(y[1])):
+                        if os.path.exists(particle_fn):
                             db.update(
-                                {'particles':z+x.strftime(y[1])},
-                                check.timestamp==x.strftime('%Y%m%d'))
+                                {'particles':particle_fn},
+                                check.timestamp==dtstr)
 
                         files_found=True
                         break
@@ -1066,7 +1060,7 @@ def nais_processor(config_file):
 
     # reprocess data in db
     if ignore_db:
-        iterator =  iter(db.search(
+        iterator = iter(db.search(
         ((check.timestamp==last_day) &
          (check.timestamp>=start_date_str) &
          (check.timestamp<=end_date_str)) |
@@ -1078,7 +1072,7 @@ def nais_processor(config_file):
 
     # do not reprocess data in db
     else:
-        iterator =  iter(db.search(
+        iterator = iter(db.search(
             ((check.timestamp==last_day) &
              (check.timestamp>=start_date_str) &
              (check.timestamp<=end_date_str)) |
@@ -1195,7 +1189,6 @@ def nais_processor(config_file):
                         check.timestamp==x['timestamp'])
 
     print("Done!")
-    os.chdir(cwd)
 
 
 def plot_nais(
@@ -1218,8 +1211,11 @@ def plot_nais(
         it saves to current directory.
 
     day : `str`
-        Date string `"%Y%m%d"` showing which
-        day to plot OR `"all"` (default) for all
+        Date string showing the day to plot
+
+        or
+        
+        `"all"` (default) for all
         days in the database.
 
     opmode : `str`
@@ -1230,42 +1226,38 @@ def plot_nais(
 
     """
 
-    # Check that the config file exists
-    if os.path.isfile(config_file) == False:
-        print('"%s" does not exist' % config_file)
-        return
-    else:
-        # Try to parse the config file
-        with open(config_file,'r') as stream:
-            try:
-                config = yaml.safe_load(stream)
-                save_path = config['processed_folder']
-                database = config['database_file']
-                location = config['location']
-            except:
-                print("Something went wrong with parsing %s",config_file)
-                return
+    if not check_config(config_file):
+        return 
 
-    # Check if you can initialize the database
-    try:
-      db = TinyDB(database)
-      check = Query()
-    except:
-        print("Could not initialize database")
-        return
+    with open(config_file,'r') as stream:
+        config = yaml.safe_load(stream)
+        save_path = config['processed_folder']
+        database = config['database_file']
+        location = config['location']
 
-    # Check if processed data path exists
-    if not os.path.exists(save_path):
-        print("Path to processed data is invalid")
-        return
+    db = TinyDB(database)
+    check = Query()
 
-    # Check if processed data path exists
     if fig_path is None:
         fig_path = os.getcwd()
-    if not os.path.exists(fig_path):
-        print("Figure save path is invalid")
+    elif os.path.exists(fig_path):
+        pass
+    else:
+        print('"fig_path" is invalid')
         return
 
+    if opmode not in ["ions","particles","both"]:
+        print('Bad "opmode"')
+
+    if quality not in ["uncleaned","cleaned","both"]:
+        print('Bad "quality"')
+
+    if day=="all":
+        pass
+    else:
+       dt=pd.to_datetime(day)
+       day=dt.strftime("%Y%m%d")
+
     # Define some plotting styles
     plt.style.use('dark_background')
 
@@ -1308,24 +1300,24 @@ def plot_nais(
 
         print('plotting %s' % x['timestamp'])
 
-        ions_exist=db.search(
+        ions_exist=bool(db.search(
             check.processed_neg_ion_file.exists() &
             check.processed_pos_ion_file.exists() &
-            (check.timestamp==x['timestamp']))
-        cleaned_ions_exist=db.search(
+            (check.timestamp==x['timestamp'])))
+        cleaned_ions_exist=bool(db.search(
             check.cleaned_processed_neg_ion_file.exists() &
             check.cleaned_processed_pos_ion_file.exists() &
-            (check.timestamp==x['timestamp']))
-        particles_exist=db.search(
+            (check.timestamp==x['timestamp'])))
+        particles_exist=bool(db.search(
             check.processed_neg_particle_file.exists() &
             check.processed_pos_particle_file.exists() &
-            (check.timestamp==x['timestamp']))
-        cleaned_particles_exist=db.search(
+            (check.timestamp==x['timestamp'])))
+        cleaned_particles_exist=bool(db.search(
             check.cleaned_processed_neg_particle_file.exists() &
             check.cleaned_processed_pos_particle_file.exists() &
-            (check.timestamp==x['timestamp']))
+            (check.timestamp==x['timestamp'])))
 
-        if (bool(ions_exist) &
+        if (ions_exist &
             ((opmode=="ions") | (opmode=="both")) &
             ((quality=="uncleaned") | (quality=="both"))):
 
@@ -1344,7 +1336,7 @@ def plot_nais(
             plt.savefig(fig_save_path,dpi=100,bbox_inches='tight')
             plt.close()
 
-        if (bool(cleaned_ions_exist) &
+        if (cleaned_ions_exist &
             ((opmode=="ions") | (opmode=="both")) &
             ((quality=="cleaned") | (quality=="both"))):
 
@@ -1363,7 +1355,7 @@ def plot_nais(
             plt.savefig(fig_save_path,dpi=100,bbox_inches='tight')
             plt.close()
 
-        if (bool(particles_exist) &
+        if (particles_exist &
             ((opmode=="particles") | (opmode=="both")) &
             ((quality=="uncleaned") | (quality=="both"))):
 
@@ -1382,7 +1374,7 @@ def plot_nais(
             plt.savefig(fig_save_path,dpi=100,bbox_inches='tight')
             plt.close()
 
-        if (bool(cleaned_particles_exist) &
+        if (cleaned_particles_exist &
             ((opmode=="particles") | (opmode=="both")) &
             ((quality=="cleaned") | (quality=="both"))):
 
@@ -1451,25 +1443,16 @@ def combine_spectra(
 
     """
 
-    if os.path.isfile(config_file) == False:
-        print('"%s" does not exist' % config_file)
-        return
-    else:
-        with open(config_file,'r') as stream:
-            try:
-                config = yaml.safe_load(stream)
-                save_path = config['processed_folder']
-                database = config['database_file']
-            except:
-                print("Something went wrong with parsing %s",config_file)
-                return
+    if not check_config(config_file):
+        return 
 
-    try:
-      db = TinyDB(database)
-      check = Query()
-    except:
-        print("Could not initialize database")
-        return
+    with open(config_file,'r') as stream:
+        config = yaml.safe_load(stream)
+        save_path = config['processed_folder']
+        database = config['database_file']
+
+    db = TinyDB(database)
+    check = Query()
 
     begin_dt=pd.to_datetime(begin_time)
     end_dt=pd.to_datetime(end_time)
